@@ -38,6 +38,7 @@
 #include <dbus/dbus.h>
 
 #include "glib-helper.h"
+#include "btio.h"
 #include "plugin.h"
 #include "logging.h"
 #include "unix.h"
@@ -54,6 +55,8 @@ static GKeyFile *load_config_file(const char *file)
 
 	keyfile = g_key_file_new();
 
+	g_key_file_set_list_separator(keyfile, ',');
+
 	if (!g_key_file_load_from_file(keyfile, file, 0, &err)) {
 		error("Parsing %s failed: %s", file, err->message);
 		g_error_free(err);
@@ -64,19 +67,29 @@ static GKeyFile *load_config_file(const char *file)
 	return keyfile;
 }
 
-static void sco_server_cb(GIOChannel *chan, int err, const bdaddr_t *src,
-			const bdaddr_t *dst, gpointer data)
+static void sco_server_cb(GIOChannel *chan, GError *err, gpointer data)
 {
 	int sk;
 	struct audio_device *device;
 	char addr[18];
+	bdaddr_t src, dst;
 
-	if (err < 0) {
-		error("accept: %s (%d)", strerror(-err), -err);
+	if (err) {
+		error("sco_server_cb: %s", err->message);
 		return;
 	}
 
-	device = manager_find_device(dst, NULL, FALSE);
+	bt_io_get(chan, BT_IO_SCO, &err,
+			BT_IO_OPT_SOURCE_BDADDR, &src,
+			BT_IO_OPT_DEST_BDADDR, &dst,
+			BT_IO_OPT_DEST, addr,
+			BT_IO_OPT_INVALID);
+	if (err) {
+		error("bt_io_get: %s", err->message);
+		goto drop;
+	}
+
+	device = manager_find_device(NULL, &src, &dst, NULL, FALSE);
 	if (!device)
 		goto drop;
 
@@ -85,8 +98,6 @@ static void sco_server_cb(GIOChannel *chan, int err, const bdaddr_t *src,
 		goto drop;
 	}
 
-	ba2str(dst, addr);
-
 	if (!get_hfp_active(device)) {
 		error("Refusing non-HFP SCO connect attempt from %s", addr);
 		goto drop;
@@ -103,8 +114,7 @@ static void sco_server_cb(GIOChannel *chan, int err, const bdaddr_t *src,
 	return;
 
 drop:
-	g_io_channel_close(chan);
-	g_io_channel_unref(chan);
+	g_io_channel_shutdown(chan, TRUE, NULL);
 }
 
 static DBusConnection *connection;
@@ -121,27 +131,38 @@ static int audio_init(void)
 
 	if (unix_init() < 0) {
 		error("Unable to setup unix socket");
-		return -EIO;
+		goto failed;
 	}
 
-	if (audio_manager_init(connection, config) < 0) {
-		dbus_connection_unref(connection);
-		return -EIO;
-	}
+	if (audio_manager_init(connection, config) < 0)
+		goto failed;
 
-	sco_server = bt_sco_listen(BDADDR_ANY, 0, sco_server_cb, NULL);
+	sco_server = bt_io_listen(BT_IO_SCO, sco_server_cb, NULL, NULL,
+					NULL, NULL,
+					BT_IO_OPT_INVALID);
 	if (!sco_server) {
 		error("Unable to start SCO server socket");
-		return -EIO;
+		goto failed;
 	}
 
 	return 0;
+
+failed:
+	audio_manager_exit();
+	unix_exit();
+
+	if (connection) {
+		dbus_connection_unref(connection);
+		connection = NULL;
+	}
+
+	return -EIO;
 }
 
 static void audio_exit(void)
 {
 	if (sco_server) {
-		g_io_channel_close(sco_server);
+		g_io_channel_shutdown(sco_server, TRUE, NULL);
 		g_io_channel_unref(sco_server);
 		sco_server = NULL;
 	}
@@ -153,4 +174,4 @@ static void audio_exit(void)
 	dbus_connection_unref(connection);
 }
 
-BLUETOOTH_PLUGIN_DEFINE("audio", audio_init, audio_exit)
+BLUETOOTH_PLUGIN_DEFINE("audio", VERSION, audio_init, audio_exit)
